from .error import WoeidError
from .modules import (
    Filters,
    Relationships
)
from .utility import Utility

BuildParams = Utility.BuildParams
BuildUrls = Utility.BuildUrls
MakeRequest = Utility.MakeRequest
PrettyPrintResult = Utility.PrettyPrintResult
GetLastRequestUrl = Utility.GetLastRequestUrl
GetLastResponseCode = Utility.GetLastResponseCode
from .api import Api



# -*- coding: utf-8 -*-
from __future__ import print_function
import woeid
from config import key
import unittest
try:
	import urlparse
	from urllib import urlencode
except ImportError:
	import urllib.parse
	urlparse = urllib.parse
	from urllib.parse import urlencode

class WoeidUnitTests(unittest.TestCase):

	def setUp(self):
		self.__api = woeid.Api(client_id=key, select='long', format='xml')
		self.__api.Lang = 'zh-hans-CN'
		self.__api.Select = 'short'
		self.__api.Format = 'json'
		self.__api.Count = 5

	def test_all(self):

		'''Example 8. Retrieving the Most Likely Place for a Given Place Name'''
		self.__api.GetPlaces(q=u'福州')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/places.q(%E7%A6%8F%E5%B7%9E);count=5;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlaces(q=u'福州') ", 'pass')

		'''Example 9. Retrieving the Five Most Likely Places for a Given Placename'''
		self.__api.GetPlaces(q=u'中国')
		self.assertEqual(woeid.GetLastRequestUrl(),'http://where.yahooapis.com/v1/places.q(%E4%B8%AD%E5%9B%BD);count=5;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlaces(q=u'中国') ", 'pass')

		'''Example 10. Retrieving All Places for a Given Place Name and Place Type'''
		self.__api.GetPlaces(q='Long Island', typ=22, nd=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/places$and(.q(Long%20Island),.type(22));count=5;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlaces(q='Long Island', typ=22, nd=True) ", 'pass')

		'''Example 11. Retrieving Places That Have the Given WOEIDs'''
		self.__api.GetPlaces(woeid=[2488042, 2488836, 2486340])
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/places.woeid(2488042,2488836,2486340);count=5;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlaces(woeid=[2488042, 2488836, 2486340]) ", 'pass')

		'''Example 12. Retrieving a Place Using a Given WOEID, in Long Representation'''
		self.__api.Select = 'long'
		self.__api.GetPlace(woeid=2507854)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2507854;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2507854) ", 'pass')

		'''Example 13. Retrieving a Place with a Given WOEID, in Short Representation'''
		self.__api.Select = 'short'
		self.__api.GetPlace(woeid=2507854)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2507854;count=5;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2507854) ", 'pass')

		'''Example 14. Retrieving a Place with a Given WOEID, in JSON format'''
		self.__api.Format = 'json'
		self.__api.GetPlace(woeid=12521721)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/12521721;count=5;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=12521721) ", 'pass')

		'''Example 15. Retrieving the Parent Place of a Given WOEID, as a Long Representation'''
		self.__api.Select = 'long'
		self.__api.GetPlace(woeid=638242, parent=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/638242/parent;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=638242, parent=True) ", 'pass')

		'''Example 16. Retrieving the Ancestors for a Given WOEID'''
		self.__api.GetPlace(woeid=12587712, ancestors=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/12587712/ancestors;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=12587712, ancestors=True) ", 'pass')

		'''Example 17. Retrieving the Belongto Places of a Given WOEID'''
		self.__api.GetPlace(woeid=23424900, belongtos=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/23424900/belongtos;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=23424900, belongtos=True) ", 'pass')

		'''Example 18. Retrieving Neighboring Places of a Given WOEID'''
		self.__api.GetPlace(woeid=2347563, neighbors=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2347563/neighbors;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2347563, neighbors=True) ", 'pass')

		'''Example 19. Retrieving Neighbors of the Neighbors of a Place With a Given WOEID'''
		self.__api.GetPlace(woeid=2347563, neighbors=True, degree=2)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2347563/neighbors.degree(2);count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2347563, neighbors=True, degree=2) ", 'pass')

		'''Example 20. Retrieving Sibling Places of a Given WOEID'''
		self.__api.GetPlace(woeid=2347563, siblings=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2347563/siblings;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2347563, siblings=True) ", 'pass')

		'''Example 21. Retrieving Children of Places of a Given WOEID'''
		self.__api.GetPlace(woeid=23424977, children=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/23424977/children;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=23424977, children=True) ", 'pass')

		'''Example 22. Retrieving Children of the Children of a Place With a Given WOEID'''
		self.__api.GetPlace(woeid=2488042, children=True, degree=2)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2488042/children.degree(2);count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2488042, children=True, degree=2) ", 'pass')

		'''Example 23. Retrieving Descendants for a Given WOEID'''
		self.__api.GetPlace(woeid=2507854, descendants=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2507854/descendants;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=2507854, descendants=True) ", 'pass')

		'''Example 24. Retrieving a Place That is a Common Ancestor of Two Places'''
		self.__api.GetPlace(woeid=(2507854, 2380824), common=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2507854/common/2380824;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=(2507854, 2380824), common=True) ", 'pass')

		'''Example 25. Retrieving a Place That is Common Ancestor of Three Places'''
		self.__api.GetPlace(woeid=(2488042, 2488836, 2486340), common=True)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/place/2488042/common/2488836/2486340;count=5;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlace(woeid=(2488042, 2488836, 2486340), common=True) ", 'pass')

		'''Example 26. Retrieving All Continents'''
		self.__api.GetContinents()
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/continents;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetContinents() ", 'pass')

		'''Example 27. Retrieving All Oceans'''
		self.__api.GetOceans()
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/oceans;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetOceans() ", 'pass')

		'''Example 28. Retrieving All Seas'''
		self.__api.GetSeas()
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/seas;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetSeas() ", 'pass')

		'''Example 29. Retrieving the Seas Adjacent to or Part of the Pacific Ocean'''
		self.__api.__select='short'
		self.__api.GetSeas(place='Pacific Ocean')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/seas/Pacific%20Ocean;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetSeas(place='Pacific Ocean') ", 'pass')

		'''Example 30. Retrieving All Countries'''
		self.__api.GetCountries()
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/countries;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetCountries() ", 'pass')

		'''Example 31. Retrieving the Countries Within North America (NA)'''
		self.__api.GetCountries(place='NA')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/countries/NA;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetCountries(place='NA') ", 'pass')

		'''Example 32. Retrieving the States Within the United States (US)'''
		self.__api.GetStates(country='US')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/states/US;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetStates(country='US') ", 'pass')

		'''Example 33. Retrieving the Counties Within California (CA)'''
		self.__api.GetCounties(state='CA')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/counties/CA;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetCounties(state='CA') ", 'pass')

		'''Example 34. Retrieving the Districts of Greater London'''
		self.__api.GetDistricts(county='Greater London')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/districts/Greater%20London;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetDistricts(county='Greater London') ", 'pass')

		'''Example 35. Retrieving the WOEID and FIPs Code for a Given ISO Code'''
		self.__api.GetConcordance(namespace='iso', id='CA-BC')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/concordance/iso/CA-BC;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetConcordance(namespace='iso', id='CA-BC') ", 'pass')

		'''Example 36. Retrieving a Collection of Place Types'''
		self.__api.GetPlacetypes()
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetypes;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetypes() ", 'pass')

		'''Example 37. Retrieving a Collection of Place Types and Their Descriptions'''
		self.__api.Select = 'long'
		self.__api.GetPlacetypes()
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetypes;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetypes() ", 'pass')

		'''Example 38. Retrieving a Partial Collection of Place Types'''
		self.__api.Select = 'short'
		self.__api.GetPlacetypes(typ=[0,2,22,37,38,15,16])
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetypes.type(0,2,22,37,38,15,16);count=0;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetypes(typ=[0,2,22,37,38,15,16]) ", 'pass')

		'''Example 39. Retrieving All Placetypes for Spain'''
		self.__api.GetPlacetypes(country='ES')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetypes/ES;count=0;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetypes(country='ES') ", 'pass')

		'''Example 40. Retrieving a Short Representation of the Resource for a Place Type'''
		self.__api.GetPlacetypes(typ=35)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetypes.type(35);count=0;start=0?format=json&select=short&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetypes(typ=35) ", 'pass')

		'''Example 41. Retrieving a Long Representation of the Resource for a Place Type'''
		self.__api.Select = 'long'
		self.__api.GetPlacetypes(typ=35)
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetypes.type(35);count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetypes(typ=35) ", 'pass')

		'''Example 42. Retrieving the Province Placetype for Spain'''
		self.__api.GetPlacetype(typ=9,country='ES')
		self.assertEqual(woeid.GetLastRequestUrl(), 'http://where.yahooapis.com/v1/placetype/9/ES;count=0;start=0?format=json&select=long&lang=zh-hans-CN&appid=dj0yJmk9NEpTYVdQSllLVmtVJmQ9WVdrOVRtVkZSWGxFTm5NbWNHbzlNQS0tJnM9Y29uc3VtZXJzZWNyZXQmeD0wMQ--')
		self.assertEqual(woeid.GetLastResponseCode(), 200)
		print("self.__api.GetPlacetype(typ=9,country='ES') ", 'pass')


if __name__ == "__main__":
	unittest.main()



from __future__ import print_function
from collections import OrderedDict
import json
import urllib
import requests
from woeid import (Relationships, Filters, WoeidError)
import xml.dom.minidom

__author__ = 'Renchen'
try:
	import urlparse
	from urllib import urlencode
except ImportError:
	import urllib.parse
	urlparse = urllib.parse
	from urllib.parse import urlencode


class ResponseCheck:
	"""A utility class reponsible for chekcing the reponse code and raise corresponding error

	Args:
		code(``int``):
			The response code in integer
	"""
	def __init__(self,
				 code):
		if code == 400:
			raise WoeidError("The appid parameter was invalid or not specified. or the q filter was missing or incorrectly specified for this resource.")

		if code == 404:
			raise WoeidError("The URI has no match in the display map")

		if code ==406:
			raise WoeidError("The requested representation is not available for this resource")

class Utility:
	__cached_url = None
	__response_code = None
	@staticmethod
	def BuildUrls(url,
			  path_elements,
			  extra_params=None,
			  extra_woeid=None,
			  filters=None,
			  relationships=None,
			  count=None,
			  start=None):
		"""An utility class reponsible for building the url string

		Args:
			path_elements(``list``(``str``)):
				A list of paths that will be appended to the base url.
			extra_params(``dict``, optional):
				A dictionary representing the parameters for making the url.
			extra_woeid(``list``(``int``) or ``list``(``str``), optional):
				This is useful when the `common` filter has been set. Aiming for making urls such as
				`1234/common/3456/73923`
			filters(``Filters``, optional):
				A `Filter` object
			relationships(``Relationships``, optional):
				A `Relationship` object
			count(``int``, optional):
				Specify the maximum number of results to return. A count of 0 is interpreted as `no maximum` (all resources)
			start(``int``, optional):
				Skip the first N results.
		Returns:
			A valid url containing all queries, filters, parameters.
		"""
		(scheme, netloc, path, params, query, fragment) = urlparse.urlparse(url)

		# Add any additional path elements to the path
		# Filter out the path elements that have a value of None
		p = [i for i in path_elements if i]
		if not path.endswith('/'):
			path += '/'
		path += '/'.join(p)

		# Add any additional query parameters to the query string
		if extra_params and len(extra_params) > 0:
			extra_query = Utility.EncodeParameters(extra_params)
			# Add it to the existing query
			if query:
				query += '&' + extra_query
			else:
				query = extra_query

		if relationships and isinstance(relationships, Relationships):
			path += str(relationships)

		if extra_woeid:
			for woeid in extra_woeid:
				path += str(woeid) + '/'
			path = path[:-1]

		if filters and isinstance(filters, Filters) and filters.IsValid():
			path += str(filters)

		if type(count) is int:
			path += ';count=%s'%str(count)

		if type(start) is int:
			path += ';start=%s'%str(start)
		# Return the rebuilt URL
		return urlparse.urlunparse((scheme, netloc, path, params, query, fragment))


	@staticmethod
	def EncodeParameters(parameters):
		"""Return a string in key=value&key=value form. Values of None are not included in the output string.

		Args:
			parameters (``OrderedDict``):
				dictionary of query parameters to be converted into a string for encoding and sending to Twitter.

		Returns:
			A URL-encoded string in "key=value&key=value" form
		"""
		if parameters is None:
			return None
		if not isinstance(parameters, OrderedDict):
			raise WoeidError("`parameters` must be a dict.")
		else:
			return urlencode(OrderedDict((k, v) for k, v in parameters.items() if v is not None))

	@staticmethod
	def BuildParams(appid,
					format='json',
					select='short',
					lang='en-us'):
		"""For constructing a parameter dictionary.

		Returns:
			Parameters dictionary
		"""
		dict = OrderedDict({})
		dict['format'] = format
		dict['select'] = select
		dict['lang'] = lang
		dict['appid'] = appid

		return dict


	@staticmethod
	def MakeRequest(url):
		"""An utility function for making url requests. Will do response check

		Returns:
			Response content in ``bytes``
		"""
		Utility.__cached_url = url
		"""An utility function for making url request
		"""
		#print("Making requests on: %s"%url)
		ret = {}
		try:
			response = requests.get(url)
			Utility.__response_code = response.status_code
			ResponseCheck(response.status_code)
			# return in bytes!
			ret = response.content
		except WoeidError as e:
			print(e.message)
			return ret
		return ret

	@staticmethod
	def PrettyPrintResult(bts):
		"""An utility function for pretty printing the result with indentation and new lines.

		Returns:
			None
		"""
		#print(isinstance(bts, bytes))
		if not isinstance(bts, bytes):
			return
		# decode into unicode
		unicode_str = bts.decode('utf-8')
		try:
			# Has to set ensure_ascii off to keep the non-ascii characters.
			print(json.dumps(json.loads(unicode_str), indent=4, separators={',' , ': '}, ensure_ascii=False))
		except TypeError as e:
			print(e)
		except ValueError as e:
			xxml = xml.dom.minidom.parseString(unicode_str)
			print(xxml.toprettyxml())
		finally:
			pass

	@staticmethod
	def GetLastRequestUrl():
		"""An utility function for fetching the most recent requesting url

		Returns:
			Last request url in ``str``
		"""
		if Utility.__cached_url:
			return Utility.__cached_url
		else:
			return None

	@staticmethod
	def GetLastResponseCode():
		"""A utility function for fetching the most recent reponse code

		Returns:
			Last response code in ``int``
		"""
		if Utility.__response_code:
			return Utility.__response_code
		else:
			return None


class WoeidError(Exception):
    """Base class for woeid errors
    """
    @property
    def message(self):
        """Returns the first argument used to construct this error.
        """
        return self.args[0]


#!/usr/bin/env python

#
#
# Copyright 2016 Renchen Sun.
#
# Licensed under the The MIT License (MIT) Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://opensource.org/licenses/MIT
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library that provides a Python interface to the Yahoo GeoPlanet API"""

from woeid import (WoeidError, Filters, Relationships, MakeRequest, BuildParams, BuildUrls)

try:
	from urllib import quote
except ImportError:
	from urllib.parse import quote

class Api(object):
	"""A python interface into the Woeid API
	 Example usage:
		 To create an instance of the woeid.Api class. Note: you will need to initialize it with your client id (consumer key) which you can obtain from Yahoo at https://developer.yahoo.com. The detailed guide can be found in the Getting Started section.

		 >>> import woeid
		 >>> api = woeid.Api(client_id='client_key')

	Args:
			client_id(``str``):
				Your yahoo client id (consumer key). Note: this is not the id of your app!
			base_url(``str``, optional):
				Specify the base url for making requests. (You probably wouldn't want to change it)
			select(``str``, optional):
				Specify how you would like your result to return. Only `short` or `long` are accepted.
			format(``str``, optional):
				Specify which format you would like to receive as a response format. Three values are accepted: `json`, `xml`, `geojson`.
			lang(``str``, optional):
				Specify the language you would like the return names to be be shown. RFC 4646 language codes are accepted.
			count(``str`` or ``int``, optional):
				Specify the maximum number of results to return. A count of 0 is interpreted as `no maximum` (all resources)
			start(``str`` or ``int``, optional):
				Skip the first N results.
	"""
	def __init__(self,
				 client_id,
				 base_url='http://where.yahooapis.com/v1/',
				 select='long',
				 format='json',
				 lang='en-us',
				 count=0,
				 start=0):
		"""Instantiate a new woeid.Api object.
		"""
		self.__base_url = base_url
		self.__clientid = client_id
		self.__format = format
		self.__select = select
		self.__lang = lang
		self.__count = count
		self.__start = start

	@property
	def BaseUrl(self):
		"""The base url

		:getter: Get the base url
		:setter: Set the base url
		:type: ``str``
		"""
		return self.__base_url

	@BaseUrl.setter
	def BaseUrl(self, url):
		try:
			self.__base_url = str(url)
		except ValueError:
			raise WoeidError("Expect a string type for url")

	@property
	def ClientId(self):
		"""The client id

		:getter: Get the client id
		:setter: Set the client id
		:type: ``str``
		"""
		return self.__clientid

	@ClientId.setter
	def ClientId(self, client_id):
		try:
			self.__clientid = str(client_id)
		except ValueError:
			raise WoeidError("Expect a string type for appid")

	@property
	def Format(self):
		"""The response format code

		:getter: Get the response format code
		:setter: Set the response format code. Supported values are: 'json', 'xml', 'geojson'
		:type: ``str``
		"""
		return self.__format

	@Format.setter
	def Format(self, format):
		try:
			self.__format = str(format)
		except ValueError:
			raise WoeidError("Expect a string type for format")

	@property
	def Select(self):
		"""The result representation mode

		:getter: Get the result representation mode
		:setter: Set the result representation mode. Supported values are: `long`, `short`
		:type: ``str``
		"""
		return self.__select

	@Select.setter
	def Select(self, select):
		try:
			if str(select).lower() == 'short' or str(select).lower() == 'long':
				self.__select = str(select)
			else:
				raise WoeidError("select can be either 'short' or 'long'")
		except ValueError:
			raise WoeidError("Expect a string type for select")

	@property
	def Lang(self):
		"""The language code (RFC 4646) used for making requests

		:getter: Get the language code used for making requests
		:setter: Set the language code used for making requests. Possible language codes are defined in RFC 4646
		:type: ``str``
		"""
		return self.__lang

	@Lang.setter
	def Lang(self, lang):
		try:
			self.__lang = str(lang)
		except ValueError:
			raise WoeidError("Expect a stirng type for lang")

	@property
	def Count(self):
		"""The format code

		:getter: Get the maximum number of results to return.
		:setter: Set the maximum number of results to return. A value of 0 is interpreted as no maximum (all resources)

		:type: ``int`` or ``str``
		"""
		return self.__count

	@Count.setter
	def Count(self, count):
		try:
			self.__count = int(count)
		except ValueError:
			raise WoeidError("Expect an integer type for count")

	@property
	def Start(self):
		"""The number of records to skip

		:getter: Get the number of records to skip
		:setter: Set the number of records to skip
		:type: ``int`` or ``str``
		"""
		return self._Start

	@Start.setter
	def Start(self, start):
		try:
			self.__start = int(start)
		except ValueError:
			raise WoeidError("Expect an integer type for start")

	def GetPlace(self,
				 woeid,
				 degree=None,
				 typ=None,
				 nd=None,
				 parent=False,
				 ancestors=False,
				 belongtos=False,
				 neighbors=False,
				 children=False,
				 siblings=False,
				 descendants=False,
				 common=False):
		"""Return a place object that matches a specified woeid or with a specified relationship specifier (parent, ancestors, belongtos, neightbors, children, siblings, descendants, common) or with filters (degree, typ, nd).

		Args:
			woeid(``str`` or ``int`` or ``tuple`` or ``list``):
				The unique place specifier.
			degree(``int`` or ``str``, optional):
				`.degree` specifier which represents the degree to which two places are neighborhoods. Only consider valid if either `neighbors` or `children` filters are set.
			typ(``str`` or ``int`` or ``list``, optional):
				`.type` specifier which is used to specify placetypes. Up to ten place types may be provided. Only consider valid if either `belongtos`, `children` or `placetypes` are set.
			nd(boolean, optional):
				`$and` specifier which is used to join two fitlers together.
			parent(``boolean``, optional):
				A relationship specifier used to return a parent place of a given woeid.
			ancestors(``boolean``, optional):
				A relationship specifier used to return one or more acestors of a place of a given woeid.
			belongtos(``boolean``, optional):
				A relationship specifier used to return a collection of places that have a place as a child or descendant (child of a child).
			neighbors(``boolean``, optional):
				A relationship specifier used to return a collection of places that neighbor of a place.
			children(``boolean``, optional):
				A relationship specifier used to return a collection of places that are children of a place.
			siblings(``boolean``, optional):
				A relationship specifier used to return a collection of places that are siblings of a place.
			descendants(``boolean``, optional):
				A relationship specifier used to return a collection of places that are in the child hierarchy (the child, the child of child, etc).
			common(``boolean``, optional):
				A relationship specifier used to return the common ancestor of both places.

		Returns:
			Response content ``in ``bytes````
		"""
		if type(woeid) == int:
			extra_paths = ['place/' + str(woeid)]
		else:
			extra_paths = ['place/' + str(woeid[0])]

		filters = Filters(typ=typ,
						  degree=degree,
						  aand=nd)

		relationships = Relationships(parent,
									 ancestors,
									 belongtos,
									 neighbors,
									 siblings,
									 children,
									 descendants,
									 common)

		extra_parms = self.__BuildParams()

		url = BuildUrls(url=self.__base_url,
						path_elements=extra_paths,
						extra_params=extra_parms,
						extra_woeid=None if type(woeid) is int else woeid[1:],
						filters=filters,
						relationships=relationships,
						count=self.__count,
						start=self.__start)
		return MakeRequest(url)

	def GetPlaces(self,
				  q=None,
				  woeid=None,
				  typ=None,
				  nd=None):

		"""
		Returns a collection of places that match a specified place name, and optionally, a specified place type. The resources in the collection are long representations of each place (unless short representations are explicitly requested).Supported Filters `.q`, `.type`, `$and`, `.woeid`.

		Args:
			q(``str`` or ``tuple``, optional):
				Specify a place name to search for or a tuple that has a place name and a focus. This filter is mutually exclusive with the `woeid` filter. The specified place can be any unicode characters. Focus can be either an ISO-3166-1 country code or a WOEID. For a "startswith" filter, specify the place as a string followed by an asterisk (*).
			woeid(``list(str)`` or ``list(int)``, optional):
				Specify a `Where On Earth Identifier` (`woeid`). Up to ten WOEIDs may be specified. This filter is mutually exclusive with the `q` filter. Example: woeid=(1,2,3)
			typ(``list(str)`` or ``list(int)`` or ``int``, optional):
				Specify one or more place type codes (https://developer.yahoo.com/geo/geoplanet/guide/concepts.html#placetypes). Up to ten place type codes or names may be provided.
			nd(``boolean``, optional):
				Specify a join operations on two filters. Example:

				 >>> import woeid
				 >>> api = woeid.Api(client_id='YOUR_CLIENT_ID')
				 >>> ret = api.GetPlaces(q='StringField', typ=22, nd=True)

		Returns:
			Response content in ``bytes``
		"""
		filters = Filters(q=q, woeid=woeid, typ=typ, aand=nd)
		extra_paths = ['places']

		extra_params = self.__BuildParams()

		url = BuildUrls(url=self.__base_url,
						path_elements=extra_paths,
						extra_params=extra_params,
						filters=filters,
						count=self.__count,
						start=self.__start)
		return MakeRequest(url)

	def __BuildParams(self):
		return BuildParams(self.__clientid,
						   format=self.__format,
						   select=self.__select,
						   lang=self.__lang)

	def __GetHelper(self,
					path=None,
					place=None,
					filters=None):
		extra_paths = [path] if type(path) is not list else path

		if type(place) is str and place:
			extra_paths.append(quote(place))

		extra_params = self.__BuildParams()

		url = BuildUrls(url=self.__base_url,
						path_elements=extra_paths,
						extra_params=extra_params,
						filters=filters,
						count=self.__start,
						start=self.__start)
		return MakeRequest(url)

	def GetContinents(self):
		"""Returns a collection of places that are continents.

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('continents')

	def GetOceans(self):
		"""Returns a collection of places that are oceans.

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('oceans')

	def GetSeas(self,
				place=None):
		"""Returns a collection of places that are seas if `place` is not provided. Returns a collection of places that are seas and are part of or adjacent to the specified continent or ocean if `place` is specified.

		Args:
			place(``str``, optional):

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('seas', place=place)

	def GetCountries(self,
					 place=None):
		"""Returns a collection of places that are countries if `place` is not provided. Returns a collection of places that are countries and are part of or adjacent to the specified continent or ocean.

		Args:
			place(``str``, optional):
				A place string

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('countries', place=place)

	def GetStates(self,
				  country):
		""" Returns a collection of places that are top-level administrative areas (states) within a country.

		Args:
			country(``str``):
				A country string

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('states', place=country)

	def GetCounties(self,
					state):
		"""Returns a collection of places that are second-level administrative areas (counties) within a top-level administrative area (state).

		Args:
			state(``str``):
				A state string.

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('counties', place=state)

	def GetDistricts(self,
					 county):
		"""Returns a collection of places that are third-level administrative areas (districts) within a second-level administrative area (county).

		Args:
			county(``str``):
				A county string

		Returns:
			Response content in ``bytes``
		"""
		return self.__GetHelper('districts', place=county)

	def GetConcordance(self,
					   namespace,
					   id):
		"""Returns information that maps dentifiers (codes) from other providers to WOEIDs. The supported namespace and id values are provided in https://developer.yahoo.com/geo/geoplanet/guide/api-reference.html#api-concordance

		Args:
			namespace(``str``):
				A namespace string
			id(``str``):
				An id string

		Returns:
			Response content in ``bytes``
		"""
		paths = ['concordance', namespace, id]
		return self.__GetHelper(paths)

	def GetPlacetypes(self,
					  country=None,
					  typ=None):
		"""Returns the complete collection of place types supported in GeoPlanet if neither `country` or `typ` is specified. Returns information about all the known placetypes. The placetype name returned will be country-specific, and may vary upon the language requested if country is specified.

		Args:
			country(``str``, optional):
				A country code string
			typ(``str`` or ``list`` or ``int``, optional):
				A type filter.

		Returns:
			Response content in ``bytes``
		"""
		filters = Filters(typ=typ)
		paths = ['placetypes']
		paths.append(country)
		return self.__GetHelper(path=paths,filters=filters)

	def GetPlacetype(self,
					 typ,
					 country):
		"""Returns a place type resource if only `typ` is specified. Returns information about a single placetype known by its code. The placetype name returned will be country-specific, and may vary depending upon the language requested.

		Args:
			typ(``str`` or ``list`` or ``int``):
				A type filter.
			country(``str``):
				A country code string

		Returns:
			Response content in ``bytes``
		"""
		paths = ['placetype']
		typ = str(typ)
		country = str(country)
		paths.append(typ)
		paths.append(country)
		return self.__GetHelper(path=paths)




__author__ = 'Renchen'
from distutils.core import setup

setup(
	name='woeid',
	version='1.0.0',
	author='Renchen Sun',
	author_email='sunrenchen@gmail.com',
	packages=['woeid', 'woeid.test'],
	url='https://github.com/Ray-SunR/woeid',
	license='LICENSE.txt',
	description='A python REST APIs for Yahoo GeoPlanet web services (https://developer.yahoo.com/geo/geoplanet/guide/)',
	long_description=open('README.rst').read(),
	install_requires=['future', 'requests', 'six'],
	keywords=['woeid', 'geoplanet', 'yahoo'],
	package_data={'': ['LICENSE.txt']}
)



try:
	from urllib import quote
except ImportError as e:
	from urllib.parse import quote
from woeid import WoeidError
from six import string_types

__author__ = 'Renchen'

class Filters:
	"""A class that encapsulates all filters
		Args:
			q(``str`` or ``tuple``, optional):
				Specify a place name to search for or a tuple that has a place name and a focus. This filter is mutually exclusive with the `woeid` filter. The specified place can be any unicode characters. Focus can be either an ISO-3166-1 country code or a WOEID. For a "startswith" filter, specify the place as a string followed by an asterisk (*).
			woeid(``list``(``str``) or ``list``(``int``), optional):
				Specify a `Where On Earth Identifier` (`woeid`). Up to ten WOEIDs may be specified. This filter is mutually exclusive with the `q` filter. Example: woeid=(1,2,3)
			typ(``list``(``str``) or ``list``(``int``) or ``int``, optional):
				Specify one or more place type codes (https://developer.yahoo.com/geo/geoplanet/guide/concepts.html#placetypes). Up to ten place type codes or names may be provided.
			degree(``int`` or ``str``, optional):
				`.degree` specifier which represents the degree to which two places are neighborhoods. Only consider valid if either `neighbors` or `children` filters are set.
			nd(``boolean``, optional):
				Specify a join operations on two filters. Example:

				 >>> import woeid
				 >>> api = woeid.Api(client_id='YOUR_CLIENT_ID')
				 >>> ret = api.GetPlaces(q='StringField', typ=22, nd=True)

		"""
	def __init__(self,
				 q=None,
				 woeid=None,
				 typ=None,
				 degree=None,
				 aand=None):
		filters = {}

		# q and woeid are mutually exclusive
		if isinstance(q, string_types) or isinstance(q, tuple):
			filters['q'] = q
		elif woeid and isinstance(woeid, list):
			# Make sure the values are str
			filters['woeid'] = [str(val) for val in woeid if isinstance(val, int) or isinstance(val, string_types)]

		if typ and isinstance(typ, list) or isinstance(typ, int):
			filters['type'] = typ

		if degree and isinstance(degree, int):
			filters['degree'] = degree

		if aand and isinstance(aand, bool):
			filters['and'] = aand
		self._filters = filters


	def HasQ(self):
		"""Return if the filter object has `.q` filter.
		"""
		return 'q' in self._filters

	def HasWoeid(self):
		"""Return if the filter object has `.woeid` filter.
		"""
		return 'woeid' in self._filters

	def HasType(self):
		"""Return if the filter object has `.type` filter
		"""
		return 'type' in self._filters

	def HasDegree(self):
		"""Return if the filter object has `.degree` filter
		"""
		return 'degree' in self._filters

	def HasAnd(self):
		"""Return if the filter object has `$and` filter
		"""
		return 'and' in self._filters

	def IsValid(self):
		return isinstance(self._filters, dict)

	def __str__(self):
		qstr = ''
		woeidstr = ''
		typestr = ''
		degreestr = ''
		andstr = ''
		# work on .q filter
		if self.HasQ():
			if isinstance(self._filters['q'], string_types):
				qstr = quote(self._filters['q'].encode('utf-8'))
			elif isinstance(self._filters['q'], tuple):
				stra = self._filters['q'][0].encode('utf-8')
				strb = self._filters['q'][1].encode('utf-8')
				# Second item will be a focus value
				# Focus can be either an ISO-3166-1 country code or a WOEID.
				qstr += quote(stra + ',' + strb)
			else:
				raise WoeidError("Unexpected usage of function! query filter is %s" % self._filters['q'])
			qstr = '.q(%s)'%qstr

		# work on .woeid filter
		if self.HasWoeid():
			if isinstance(self._filters['woeid'], list) and len(self._filters[
			'woeid']) > 1:
				for item in self._filters['woeid']:
					if (isinstance(item, string_types) and item.isdigit()) or isinstance(item, int):
						woeidstr += quote(item) + ','
				# tick out the last comma
				woeidstr = woeidstr[:-1]
			elif isinstance(self._filters['woeid'], list) and len(
					self._filters['woeid']) == 1:
				woeidstr = '/' + quote(self._filters['woeid'][0])
			else:
				raise WoeidError("Unexpected usage of function! query filter is %s"%self._filters['woeid'])
			#.woeid can be omitted if there is only one item
			if ',' in woeidstr:
				woeidstr = '.woeid(%s)'%woeidstr

		# work on .type filter
		if 'type' in self._filters:
			tpitem = self._filters['type']
			if isinstance(tpitem, list):
				for item in tpitem:
					if (isinstance(item, string_types) and item.isdigit()) or isinstance(item, int):
						typestr += quote(str(item)) + ','
				typestr = typestr[:-1]
				typestr = '.type(%s)'%typestr
			elif (type(tpitem) is str and tpitem.isdigit()) or isinstance(tpitem, int):
				typestr = '.type(%s)'%quote(str(tpitem))

		# work on .degree filter
		if 'degree' in self._filters:
			degree = str(self._filters['degree'])
			degreestr = '.degree(%s)'%degree

		# work on .and filter
		if 'and' in self._filters:
			conda = ''
			condb = ''
			if self.HasQ() and qstr:
				conda = qstr
			if self.HasWoeid() and woeidstr:
				conda = woeidstr

			if typestr:
				condb = typestr
			if degreestr:
				condb = degreestr

			if conda and condb:
				andstr = '$and(%s,%s)'%(conda,condb)

		if andstr:
			return andstr


		query_or_woeid_str = qstr if qstr else woeidstr

		return query_or_woeid_str + typestr + degreestr


class Relationships:
	""""A class that encapsulates all relationships

		Args:
			parent(``boolean``, optional):
				A relationship specifier used to return a parent place of a given woeid.
			ancestors(``boolean``, optional):
				A relationship specifier used to return one or more acestors of a place of a given woeid.
			belongtos(``boolean``, optional):
				A relationship specifier used to return a collection of places that have a place as a child or descendant (child of a child).
			neighbors(``boolean``, optional):
				A relationship specifier used to return a collection of places that neighbor of a place.
			children(``boolean``, optional):
				A relationship specifier used to return a collection of places that are children of a place.
			siblings(``boolean``, optional):
				A relationship specifier used to return a collection of places that are siblings of a place.
			descendants(``boolean``, optional):
				A relationship specifier used to return a collection of places that are in the child hierarchy (the child, the child of child, etc).
			common(``boolean``, optional):
				A relationship specifier used to return the common ancestor of both places.
		"""
	def __init__(self,
				 parent=False,
				 ancestors=False,
				 belongstos=False,
				 neighbors=False,
				 siblings=False,
				 children=False,
				 descendants=False,
				 common=False):
		self._parent=parent
		self._ancestors=ancestors
		self._belongtos=belongstos
		self._neighbors=neighbors
		self._siblings=siblings
		self._children=children
		self._descendants=descendants
		self._common=common

	def __str__(self):
		if self._parent:
			return '/parent'

		if self._ancestors:
			return '/ancestors'

		if self._belongtos:
			return '/belongtos'

		if self._neighbors:
			return '/neighbors'

		if self._siblings:
			return '/siblings'

		if self._children:
			return '/children'

		if self._descendants:
			return '/descendants'

		if self._common:
			return '/common/'

		return ''

	def __Validate(self, filters):
		if isinstance(filters, Filters):
			raise WoeidError("Unexpected modules usage: %s"%"Validate takes a Filters object as its argument")

		if not filters.IsValid():
			raise WoeidError("Unexpected API usage: %s"%"filters should be a dictionary")

		''' /parent, /ancestors, /siblings, /common/ don't support any filters'''
		if self._parent and filters.keys():
			raise WoeidError("Unexpected API usage: %s"%"woeid/parent doesn't support filters")

		if self._ancestors and filters.keys():
			raise WoeidError("Unexpected API usage: %s"%"woeid/ancestors doesn't support filters")

		if self._siblings and filters.keys():
			raise WoeidError("Unexpected API usage: %s"%"woeid/siblings doesn't support filters")

		if self._common and filters.keys():
			raise WoeidError("Unexpected API usage: %s"%"woeid1/common/woeid2 doesn't support filters")

		'''/belongtos and /descendants and /children support .type filter'''
		if self._belongtos and (filters.HasAnd() or filters.HasDegree() or filters.HasQ() or filters.HasWoeid()):
			raise WoeidError("Unexpected API usage: %s"%"woeid/belongtos supports .type filter only")

		if self._descendants and (filters.HasAnd() or filters.HasDegree() or filters.HasQ() or filters.HasWoeid()):
			raise WoeidError("Unexpected API usage: %s"%"woeid/descendants supports .type filter only")

		if self._children and (filters.HasWoeid() or filters.HasQ() or filters.HasAnd()):
			raise WoeidError("Unexpected API usage: %s"%"woeid/children supports .degree or .type filters only")

		'''/neighbors support .degree filter'''
		if self._neighbors and (filters.HasWoeid() or filters.HasType() or filters.HasQ() or filters.HasAnd()):
			raise WoeidError("Unexpected API usage: %s"%"woeid/neighbors supports .degree filter only")







